<?php

namespace Drupal\frontend_editing\Twig\Extension;

use Drupal\Core\Controller\ControllerResolverInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\twig_field_value\Twig\Extension\FieldValueExtension as TwigFieldValueExtension;

/**
 * Extends TwigFieldValueExtension to add frontend editing support.
 */
class FieldValueExtension extends TwigFieldValueExtension {

  /**
   * The decorated service.
   *
   * @var \Drupal\twig_field_value\Twig\Extension\FieldValueExtension
   */
  protected TwigFieldValueExtension $inner;

  /**
   * Constructs a FieldValueExtension.
   *
   * @param \Drupal\twig_field_value\Twig\Extension\FieldValueExtension $inner
   *   The service that is being decorated.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   * @param \Drupal\Core\Entity\EntityRepositoryInterface $entityRepository
   *   The entity repository.
   * @param \Drupal\Core\Controller\ControllerResolverInterface $controllerResolver
   *   The controller resolver.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $loggerFactory
   *   The logger channel factory.
   */
  public function __construct(TwigFieldValueExtension $inner, LanguageManagerInterface $language_manager, EntityRepositoryInterface $entityRepository, ControllerResolverInterface $controllerResolver, LoggerChannelFactoryInterface $loggerFactory) {
    parent::__construct($language_manager, $entityRepository, $controllerResolver, $loggerFactory);
    $this->inner = $inner;
  }

  /**
   * {@inheritdoc}
   */
  public function getFieldLabel($build) {
    $this->findFieldRenderArray($build);
    return parent::getFieldValue($build);
  }

  /**
   * {@inheritdoc}
   */
  public function getRawValues($build, $key = '') {
    $this->findFieldRenderArray($build);
    return parent::getRawValues($build, $key);
  }

  /**
   * {@inheritdoc}
   */
  public function getTargetEntity($build) {
    $this->findFieldRenderArray($build);
    return parent::getTargetEntity($build);
  }

  /**
   * Searches for field render array in the given array.
   *
   * Frontend editing render arrays for entity reference (revisions) fields
   * contain more than one element to have the reliable field wrapper, so the
   * default static::isFieldRenderArray() check will always fail. For that
   * reason we need to search for at least one element from the render array
   * that contains an element with '#theme' "field" to be sure that the field is
   * being rendered.
   *
   * @param array|null $build
   *   The render array.
   */
  private function findFieldRenderArray(&$build) {
    // Make sure that the given array is from frontend editing.
    if (is_array($build) && isset($build[0])) {
      foreach ($build as $element) {
        if (is_array($element) && isset($element['#theme']) && $element['#theme'] == 'field') {
          $build = $element;
          break;
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function isFieldRenderArray($build) {
    $test_build = $build;
    $this->findFieldRenderArray($test_build);
    return isset($test_build['#theme']) && $test_build['#theme'] == 'field';
  }

}
